SystemOrganization addCategory: #StartupPreferences!

!DosFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 3/7/2012 19:21'!
preferencesFolder

	^ self forFileName: ( self preferencesRootPath , SystemVersion current dottedMajorMinor , self slash )


! !

!DosFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminTerrenceComan 2/12/2012 14:22'!
preferencesGeneralFolder

	^ self forFileName: ( self preferencesRootPath, 'general' , self slash )
	

! !

!DosFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenComan 4/15/2012 00:40'!
preferencesRootPath
	"For further details refer to http://code.google.com/p/pharo/issues/detail?id=5255" 

	|  osMajorVer osMinorVer rootPath exeFolderList  fallbackRootPath |
	
	"First try to adhere to hardcoded Microsoft conventions - though we really should be reading environment variables since %SYSTEMDRIVE% might not be C: "
	
       osMajorVer := Smalltalk os osVersion first.   
       osMinorVer := Smalltalk os osVersion third.
       osMajorVer >= $6 ifTrue: [ rootPath := 'C:\ProgramData\Pharo\' ].   "Vista, 2008, 7 and later"  
       osMajorVer = $5 ifTrue: [ rootPath := 'C:\Documents and Settings\All Users\Application Data\Pharo\' ]. "2000, XP, 2003"
       osMajorVer = $4 ifTrue: 
       [   rootPath := 'C:\Windows\All Users\Application Data\Pharo\' . "98, 98SE, Me"
           osMinorVer = $0 ifTrue: [ rootPath := 'C:\WinNT\Profiles\All Users\Application Data\Pharo\' ] .    "NT 4"
       ].
       osMajorVer = $3 ifTrue: [ rootPath := 'C:\WinNT\Profiles\All Users\Application Data\Pharo\' ] . "NT 3.1 3.5 3.51"	

	"An alternative fall back folder in case system locked down such that user cannot follow Microsoft conventions above, or is using a portable drive on a new system"	
	exeFolderList := (self default fullName subStrings: self slash) copyFrom: 1 to: 2.
	fallbackRootPath := (exeFolderList joinUsing: self slash) , '\_config\Pharo\' .

	"Now work out which rootPath user is using.  Perhaps we should inform users of their specific options"	
	rootPath ifNotNil: [ ( self forFileName: rootPath ) exists ifTrue: [ ^rootPath ] ].
	fallbackRootPath ifNotNil: [ ( self forFileName: fallbackRootPath ) exists ifTrue: [ ^fallbackRootPath ] ].
	^ '\'. "Avoids later MNU"
	
	

! !

!MacFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 3/7/2012 19:21'!
preferencesFolder

	| folderStringName list parentPath path directory |
	folderStringName := '.config/pharo/',SystemVersion current dottedMajorMinor.
	list := (self default fullName subStrings: self slash) copyFrom: 1 to: 2.
	parentPath := list joinUsing: self slash.
	parentPath := self slash,parentPath, self slash.
	path := parentPath, folderStringName,self slash.
	directory := self forFileName: path.
		
	^ directory! !

!MacFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 11/21/2011 17:23'!
preferencesGeneralFolder

	| folderStringName list parentPath path directory |
	folderStringName := '.config/pharo/general'.
	list := (self default fullName subStrings: self slash) copyFrom: 1 to: 2.
	parentPath := list joinUsing: self slash.
	parentPath := self slash,parentPath, self slash.
	path := parentPath, folderStringName,self slash.
	directory := self forFileName: path.
		
	^ directory! !

Object subclass: #PreferencesHandler
	instanceVariableNames: 'firstLink actions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StartupPreferences'!

!PreferencesHandler commentStamp: '<historical>' prior: 0!
I am the one which manage the links of the chain of responsibilites to retreive the good preference file!

!PreferencesHandler methodsFor: 'initialization' stamp: 'EstebanLorenzano 7/10/2012 10:58'!
buildActionList
    "First, we look at image folder,
    then we look at preferences folder,
    and finally at general preferences folder "

    | directories |
	self flag: #todo. "I'm disabling this sequence because lookup of preferences 
	folder is broken in windows.
	NOTE: this has been fixed in Pharo 2.0"
	directories := OrderedCollection new.
	directories add: [ self lookInImageDirectory ].
	(OSPlatform current platformFamily = #Windows) 
		ifFalse: [
			directories add: [self lookInPreferencesFolder ].
			directories add: [self lookInGeneralPreferencesFolder ] ].
	^ directories! !

!PreferencesHandler methodsFor: 'initialization' stamp: 'IgorStasenko 2/4/2012 18:50'!
initialize
	"Initialization code for PreferencesHandler"

	super initialize.
	
	actions := self buildActionList.! !

!PreferencesHandler methodsFor: 'actions' stamp: 'SeanDeNigris 7/12/2012 13:37'!
lookInGeneralPreferencesFolder

	| result folder scripts |

	result := OrderedCollection new.
	folder := FileDirectory preferencesGeneralFolder.
	folder exists ifFalse: [ ^ nil ].
	
	scripts := folder matchingEntries: '*.st'.
	scripts do: [ :file |
		result add: (folder fileNamed: file name)].

	^ result isEmpty
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!PreferencesHandler methodsFor: 'actions' stamp: 'BenjaminVanRyseghem 2/4/2012 22:02'!
lookInImageDirectory

	| folder |
	
	folder := FileDirectory default.
	folder exists ifFalse: [ ^ nil ].
	
	^ (folder fileExists: 'startup.st')
			ifTrue:	[ { folder fileNamed: 'startup.st' }]
			ifFalse: [ nil ]! !

!PreferencesHandler methodsFor: 'actions' stamp: 'SeanDeNigris 7/12/2012 13:38'!
lookInPreferencesFolder

	| result folder scripts |
		
	result := OrderedCollection new.
	
	folder := FileDirectory preferencesFolder.
	folder exists ifFalse: [ ^ nil ].
	
	scripts := folder matchingEntries: '*.st'.
	scripts do: [ :file | result add: (folder fileNamed: file name)].
	
	^ result isEmpty
		ifTrue: [ nil ]
		ifFalse: [ result ]
	! !

!PreferencesHandler methodsFor: 'protocol' stamp: 'IgorStasenko 2/4/2012 18:34'!
perform

	actions do: [:each |
		
		each value ifNotNil: [:res | ^ res ]
	].
		
	^#()
	
! !

Object subclass: #StartupAction
	instanceVariableNames: 'code hasBeenExecuted runOnce nameOfItem name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StartupPreferences'!

!StartupAction commentStamp: '<historical>' prior: 0!
I am an atomic item used to execute a portion of code at image startup.!

!StartupAction class methodsFor: 'instance creation' stamp: 'IgorStasenko 2/4/2012 19:09'!
name: nameOfItem code: code

	^ self new
		code: code;
		name: nameOfItem;
		yourself! !

!StartupAction class methodsFor: 'instance creation' stamp: 'IgorStasenko 2/4/2012 19:09'!
name: nameOfItem code: code runOnce: aBoolean

	^ self new
		code: code;
		name: nameOfItem;
		runOnce: aBoolean;
		yourself! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
= anObject

	anObject class = self class ifFalse: [ ^ false ].
	^ self code = anObject code and: [ self name = anObject name and: [ self runOnce = anObject runOnce ]]! !

!StartupAction methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 11/1/2011 09:12'!
addAtStartup

	StartupLoader default addAtStartup: { self }! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:06'!
code

	^ code! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:05'!
code: anObject
	
	code := anObject! !

!StartupAction methodsFor: 'execution' stamp: 'SeanDeNigris 7/7/2012 23:42'!
execute

	| block correctlyExecuted |
	(self hasBeenExecuted and: [ self runOnce ]) ifTrue: [ ^ self ].
	block := self code isBlock
				ifTrue: [ self code ]
				ifFalse: [[ self class compilerClass evaluate: self code ]].
	correctlyExecuted := true. 
	block on: Halt,Error do: [:ex | StartupLoader default errors add: {ex. self}. correctlyExecuted := false].
	self hasBeenExecuted: correctlyExecuted.! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:06'!
hasBeenExecuted

	^ hasBeenExecuted ifNil: [ hasBeenExecuted := false ]! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:05'!
hasBeenExecuted: anObject
	
	hasBeenExecuted := anObject! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
name

	^ name! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
name: anObject
	
	name := anObject! !

!StartupAction methodsFor: 'printing' stamp: 'IgorStasenko 2/4/2012 19:07'!
printOn: aStream

	aStream <<'a ' << self class name << '(' << self name << ')'! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 18:10'!
runOnce

	^ runOnce ifNil: [ runOnce := false ]! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 18:11'!
runOnce: anObject
	
	runOnce := anObject! !

!StartupAction methodsFor: 'printing' stamp: 'IgorStasenko 2/4/2012 19:07'!
storeOn: aStream

	aStream << self class name << ' name: '<< self name printString <<' code: '<< self code printString.
	self runOnce ifTrue: [ aStream << ' runOnce: true']! !

Object subclass: #StartupLoader
	instanceVariableNames: 'errors actions'
	classVariableNames: 'AllowStartupScript UniqueInstance'
	poolDictionaries: ''
	category: 'StartupPreferences'!

!StartupLoader commentStamp: '<historical>' prior: 0!
I am in charge of the loading of the startup scripts from ~/.config/pharo/#{version}/startup.st or ~/.config/pharo/general/pharo#{version}.st

StartupLoader example

will define a script sample startup.st in your unix root on unix 

Its contents is 

StartupLoader default executeAtomicItems: {
	StartupAtomicItem name: 'Open Help' code: 'Workspace openContents: ''Here is just an example of how to use the StartupLoader.
I should only be displayed once.
	
You can also see StartupLoader class>>#example'' label: ''Help''' isSingleton: true.
	StartupAtomicItem name: 'Open Workspace' code: 'Workspace openContents: ''I should be displayed each time'''.
}

For a more complete example, see StartupLoader class>>#example2!

!StartupLoader class methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:17'!
allowStartupScript

	^ AllowStartupScript ifNil: [ AllowStartupScript := true ]! !

!StartupLoader class methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:17'!
allowStartupScript: aBoolean

	AllowStartupScript := aBoolean! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 08:56'!
default

	^ UniqueInstance ifNil: [ UniqueInstance := super basicNew initialize ]! !

!StartupLoader class methodsFor: 'example' stamp: 'IgorStasenko 2/4/2012 18:09'!
example
	" an example of script generation "
	| item1 item2 |
	item1 := StartupAction name: 'Open Help' code: 'Workspace openContents: ''Here is just an example of how to use the StartupLoader.
I should only be displayed once.
	
 You can also see StartupLoader class>>#example'' label: ''Help''' runOnce: true.
	item2 := StartupAction name: 'Open Workspace' code:  [ Workspace openContents: 'I should be displayed each time' ].
	StartupLoader default addAtStartupForGeneral: {item1. item2}.
	
	StartupLoader default loadStartupScript.
	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'MarcusDenker 2/6/2012 15:43'!
example2
	" it's my personal script provided as example"
	"self example2"
	| items |
	items := OrderedCollection new.
	items add: (StartupAction name: 'Set the Author' code: [ Author fullName: Author fullName printString ]). "replace it by your name"
	
	StartupLoader default addAtStartupForGeneral: items named: 'author.st'.
	items removeAll.
	
	items add: (StartupAction name: 'Debugger option' code: [ Debugger alwaysOpenFullDebugger: true ]).
	items add: (StartupAction name: 'Dragging Option' code: [ UITheme defaultSettings fastDragging: true ]).
	items add:  (StartupAction name: 'CMD Left Shortcut' code: [ TasklistMorph useHistoricalShortcuts: false ]).
	items add: (StartupAction name: 'Dialog Auto Accept' code: [ TextEditorDialogWindow autoAccept: true ]).
	StartupLoader default addAtStartupForGeneral: items named: 'settings.st'.
	items removeAll.
	
	items add: (StartupAction name: 'Fonts option' code: [ FreeTypeSystemSettings loadFt2Library: true.
FreeTypeFontProvider current updateFromSystem.
StandardFonts defaultFont: (LogicalFont familyName: 'Lucida Grande' pointSize: 10) forceNotBold.
GraphicFontSettings resetAllFontToDefault.
StandardFonts codeFont: (LogicalFont familyName: 'Consolas' pointSize: 10).] runOnce: true).
	
	StartupLoader default addAtStartupForGeneral: items named: 'fonts.st'.
	items removeAll.
	StartupLoader default addAtStartup: items.
	
	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'IgorStasenko 2/4/2012 18:08'!
example3
	" it's my personal script provided as example"
	"self example3"
	| items |
	items := OrderedCollection new.
	items add: (StartupAction name: 'Self halt' code: [ self halt ]). "replace it by your name"
	items add: (StartupAction name: 'WS' code: [ Workspace open ]). "replace it by your name"
	StartupLoader default addAtStartupForGeneral: items named: 'author.st'.

	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'initialize' stamp: 'SeanDeNigris 8/14/2012 12:12'!
initialize

	super initialize.
	Smalltalk addToStartUpList: self before: CommandLine.! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 08:56'!
new

	^ self error: 'use #default instead'! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 09:14'!
reset

	UniqueInstance := nil! !

!StartupLoader class methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 2/10/2012 18:44'!
startUp: resuming

	(self allowStartupScript and: [ resuming ])
		ifTrue: [ Smalltalk addDeferredStartupAction: [ self default loadStartupScript ]]! !

!StartupLoader class methodsFor: 'menu' stamp: 'MarcusDenker 3/24/2012 17:07'!
startupLoaderMenuOn: aBuilder 
	<worldMenu>
	
	(aBuilder item: #startupLoader)
		action: [ self default loadStartupScript ];
		label: 'Run startup scripts';
		parent: #System;
		order: 2;
		help: 'Run startup "scripts';
		icon: UITheme current smallJustifiedIcon! !

!StartupLoader class methodsFor: 'setting' stamp: 'IgorStasenko 2/4/2012 19:17'!
startupPreferencesSettingsOn: aBuilder
	<systemsettings>
	
	(aBuilder group: #'Startup Preferences')
		target: self;
		label: 'Startup Preferences' translated;
		description: 'Settings related to Startup Preferences' translated;
		with: [
			(aBuilder setting: #allowStartupScript)
				label: 'Allow the preferences to be loaded at startup' translated;
				description: 'If true, the preferences will be loaded' translated ]! !

!StartupLoader methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:06'!
actions

	^ actions! !

!StartupLoader methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:08'!
add: anAction

	^ actions at: anAction name ifAbsentPut: [ anAction ]! !

!StartupLoader methodsFor: 'script generation' stamp: 'BenjaminVanRyseghem 11/21/2011 18:02'!
addAtStartup: aCollection

	| preferencesFolder |
	preferencesFolder := FileDirectory preferencesFolder.
	preferencesFolder assureExistence.
	
	preferencesFolder forceNewFileNamed: self startupScriptName do: [:stream | stream nextPutAll: (self buildStreamFor: aCollection) ].! !

!StartupLoader methodsFor: 'script generation' stamp: 'BenjaminVanRyseghem 11/24/2011 09:28'!
addAtStartupForGeneral: aCollection

	self addAtStartupForGeneral: aCollection named: self startupGeneralScriptName! !

!StartupLoader methodsFor: 'script generation' stamp: 'BenjaminVanRyseghem 12/13/2011 11:28'!
addAtStartupForGeneral: aCollection named: aString

	| preferencesGeneralFolder preferencesFolder |	
	preferencesGeneralFolder := FileDirectory preferencesGeneralFolder.
	preferencesGeneralFolder assureExistence.
	
	preferencesFolder := FileDirectory preferencesFolder.
	preferencesFolder assureExistence.
	
	preferencesGeneralFolder forceNewFileNamed: aString do: [:stream | stream nextPutAll: (self buildStreamFor: aCollection) ].
	preferencesFolder forceNewFileNamed: aString do: [:stream | stream nextPutAll: (self buildStreamFor: aCollection) ].! !

!StartupLoader methodsFor: 'script generation' stamp: 'BenjaminVanRyseghem 12/14/2011 13:42'!
buildStreamFor: aCollection
	| stream |
	
	stream := WriteStream on: ''.

	stream
		nextPutAll: 'StartupLoader default executeAtomicItems: {';
		nextPutAll: String crlf.
	aCollection do: [:item |
		stream 
			nextPut: Character tab;
			nextPutAll: item storeString;
			nextPut: $.;
			nextPutAll: String crlf ].
	stream
		nextPut: $};
		nextPut: $.;
		nextPutAll: String crlf;
		nextPutAll: String crlf.
	^ stream contents! !

!StartupLoader methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:45'!
errors

	^ errors! !

!StartupLoader methodsFor: 'code execution' stamp: 'BenjaminVanRyseghem 3/20/2012 11:18'!
executeAtomicItems: aCollection
	"No sender but send from the generated script"

	'Load Startup Preferences'
		displayProgressFrom: 0 to: aCollection size
		during: [:bar | 
			aCollection doWithIndex: [:item :index | 
				bar value: index.
				(self add: item) execute ]]! !

!StartupLoader methodsFor: 'initialize' stamp: 'IgorStasenko 2/4/2012 19:11'!
initialize

	super initialize.
	actions := Dictionary new. 
	errors := OrderedCollection new. ! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:29'!
loadPreferences

	self retreiveFilesStream do: [:file | file fileIn ].! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:28'!
loadStartupScript

	errors removeAll.
	self loadPreferences.! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:29'!
retreiveFilesStream

	^ PreferencesHandler new perform! !

!StartupLoader methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/7/2012 19:21'!
startupGeneralScriptName

	^ 'pharo',SystemVersion current dottedMajorMinor,'.st'! !

!StartupLoader methodsFor: 'private' stamp: 'BenjaminVanRyseghem 11/1/2011 09:15'!
startupScriptName

	^ 'startup.st'! !

!UnixFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 3/7/2012 19:22'!
preferencesFolder

	| folderStringName list parentPath path directory |
	folderStringName := '.config/pharo/',SystemVersion current dottedMajorMinor.
	list := (self default fullName subStrings: self slash) copyFrom: 1 to: 2.
	parentPath := list joinUsing: self slash.
	parentPath := self slash,parentPath, self slash.
	path := parentPath, folderStringName,self slash.
	directory := self forFileName: path.
		
	^ directory! !

!UnixFileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 11/21/2011 17:23'!
preferencesGeneralFolder

	| folderStringName list parentPath path directory |
	folderStringName := '.config/pharo/general'.
	list := (self default fullName subStrings: self slash) copyFrom: 1 to: 2.
	parentPath := list joinUsing: self slash.
	parentPath := self slash,parentPath, self slash.
	path := parentPath, folderStringName,self slash.
	directory := self forFileName: path.
		
	^ directory! !

!FileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 11/1/2011 08:33'!
preferencesFolder

	^ DirectoryClass preferencesFolder! !

!FileDirectory class methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 11/21/2011 17:23'!
preferencesGeneralFolder

	^ DirectoryClass preferencesGeneralFolder! !
StartupLoader initialize!
